﻿Imports System.Text
Imports System.Threading.Tasks

Namespace SolSoft.DataBinding.WpfDemo
	''' <summary>
	''' Creates a delegate command whose 'can execute' state is tied to the given property
	''' Note that this creates a reference to that property (via the ValueChanged event) so the objects should have similar lifetimes
	''' Or use dispose
	''' </summary>
	Friend Class DelegateCommandFromProperty
		Inherits DelegateCommand
		Implements IDisposable
		Private m_canExecuteProperty As IProperty(Of Boolean)

		Public Sub New(ByVal canExecute As IProperty(Of Boolean), ByVal execute As Action)
			MyBase.New(CreateCanExecuteDelegate(canExecute), execute)
			m_canExecuteProperty = canExecute

			'wire 'can execute' to the given boolean property
			AddHandler m_canExecuteProperty.ValueChanged, AddressOf canExecuteProperty_ValueChanged
		End Sub

		Private Shared Function CreateCanExecuteDelegate(ByVal canExecute As IProperty(Of Boolean)) As Func(Of Boolean)
			If canExecute Is Nothing Then
				Throw New ArgumentNullException(ObjectNamingExtensions.GetName(Function() canExecute))
			End If
			Return Function() canExecute.Value
		End Function

		Private Sub canExecuteProperty_ValueChanged(ByVal sender As Object, ByVal e As ValueChangedEventArgs)
			OnCanExecuteChanged(EventArgs.Empty)
		End Sub


		Public Sub Dispose() Implements IDisposable.Dispose
			Dispose(True)
			GC.SuppressFinalize(Me)
		End Sub

		Protected Overridable Sub Dispose(ByVal disposing As Boolean)
			If disposing Then
				'dispose of managed resources:
				RemoveHandler m_canExecuteProperty.ValueChanged, AddressOf canExecuteProperty_ValueChanged
			End If

			'dispose of unmanaged resources:
		End Sub
	End Class
End Namespace
