﻿Imports System.Text
Imports System.Threading.Tasks

Namespace SolSoft.DataBinding.WpfDemo
	Public Class MainViewModel
		Inherits BaseViewModel
		Public Sub New()

			Me.m_value1Property = Me.CreateNotifyProperty(Function() Value1)
			Me.m_value2Property = Me.CreateNotifyProperty(Function() Value2)
			Me.m_value3Property = Me.CreateNotifyProperty(Function() Value3)
			Me.m_value4Property = Me.CreateNotifyProperty(Function() Value4)
			Me.m_totalValueProperty = Me.CreateDerivedNotifyProperty(Function() TotalValue, m_value1Property, m_value2Property, m_value3Property, m_value4Property, AddressOf CalculateTotal)

			Me.m_isBusyProperty = IsBusyNotifyProperty.Create(Me, Function() IsBusy)
			Me.m_isEnabledProperty = Me.CreateDerivedNotifyProperty(Function() IsEnabled, m_isBusyProperty, Function(isBusy) (Not isBusy))

			Me.m_generateValuesCommand = New DelegateCommandFromProperty(m_isEnabledProperty, AddressOf GenerateValues)
			Me.m_clearValuesCommand = New DelegateCommandFromProperty(m_isEnabledProperty, AddressOf ClearValues)

			Me.m_quitCommand = New DelegateCommand(Function() True, AddressOf RequestClose)
		End Sub

		#Region "Values"
		Private ReadOnly m_value1Property As NotifyProperty(Of Integer?)
		Public Property Value1() As Integer?
			Get
				Return m_value1Property.Value
			End Get
			Set(ByVal value? As Integer)
				m_value1Property.SetValue(value)

			End Set
		End Property

		Private ReadOnly m_value2Property As NotifyProperty(Of Integer?)
		Public Property Value2() As Integer?
			Get
				Return m_value2Property.Value
			End Get
			Set(ByVal value? As Integer)
				m_value2Property.SetValue(value)

			End Set
		End Property

		Private ReadOnly m_value3Property As NotifyProperty(Of Integer?)
		Public Property Value3() As Integer?
			Get
				Return m_value3Property.Value
			End Get
			Set(ByVal value? As Integer)
				m_value3Property.SetValue(value)

			End Set
		End Property

		Private ReadOnly m_value4Property As NotifyProperty(Of Integer?)
		Public Property Value4() As Integer?
			Get
				Return m_value4Property.Value
			End Get
			Set(ByVal value? As Integer)
				m_value4Property.SetValue(value)

			End Set
		End Property

		#End Region

		#Region "Total"

		Private ReadOnly m_totalValueProperty As DerivedNotifyProperty(Of Integer?)

		''' <summary>
		''' Calculates the total, if any, from the given values
		''' </summary>
		Private Function CalculateTotal(ByVal value1? As Integer, ByVal value2? As Integer, ByVal value3? As Integer, ByVal value4? As Integer) As Integer?
			Dim toReturn? As Integer = Nothing

			If value1 IsNot Nothing Then
				toReturn = (If(toReturn, 0)) + value1
			End If

			If value2 IsNot Nothing Then
				toReturn = (If(toReturn, 0)) + value2
			End If

			If value3 IsNot Nothing Then
				toReturn = (If(toReturn, 0)) + value3
			End If

			If value4 IsNot Nothing Then
				toReturn = (If(toReturn, 0)) + value4
			End If

			Return toReturn
		End Function
		Public ReadOnly Property TotalValue() As Integer?
			Get
				Return m_totalValueProperty.Value
			End Get
		End Property
		#End Region

		#Region "Busy/Enabled"
		Private ReadOnly m_isBusyProperty As IsBusyNotifyProperty
		''' <summary>
		''' Determines whether the UI is busy
		''' </summary>
		Public Property IsBusy() As Boolean
			Get
				Return m_isBusyProperty.Value
			End Get
			Protected Set(ByVal value As Boolean)
				m_isBusyProperty.SetValue(value)
			End Set
		End Property

		Private ReadOnly m_isEnabledProperty As DerivedNotifyProperty(Of Boolean)
		''' <summary>
		''' Determines whether the UI is enabled
		''' </summary>
		Public ReadOnly Property IsEnabled() As Boolean
			Get
				Return m_isEnabledProperty.Value
			End Get
		End Property
		#End Region

		''' <summary>
		''' Generates a random number for each value, with a delay in between to simulate processing
		''' </summary>
		Private async Sub GenerateValues()
			Dim r As New Random()

			IsBusy = True
			Try
				Dim delay = TimeSpan.FromSeconds(.66)
                Const min As Integer = 5
                Const max As Integer = 15

				await Task.Delay(delay)
				Value1 = r.Next(min, max)

				await Task.Delay(delay)
				Value2 = r.Next(min, max)

				await Task.Delay(delay)
				Value3 = r.Next(min, max)

				await Task.Delay(delay)
				Value4 = r.Next(min, max)
			Finally
				IsBusy = False
			End Try
		End Sub

		Private ReadOnly m_generateValuesCommand As DelegateCommand
		Public ReadOnly Property GenerateValuesCommand() As ICommand
			Get
				Return m_generateValuesCommand
			End Get
		End Property

		''' <summary>
		''' Clears out the values by sequentially counting them down to 0
		''' </summary>
		Private async Sub ClearValues()
			IsBusy = True
			Try
				Dim delay = TimeSpan.FromSeconds(.2)

				Do While Value1 > 0
					await Task.Delay(delay)
					Value1 -= 1
				Loop
				Value1 = Nothing

				Do While Value2 > 0
					await Task.Delay(delay)
					Value2 -= 1
				Loop
				Value2 = Nothing

				Do While Value3 > 0
					await Task.Delay(delay)
					Value3 -= 1
				Loop
				Value3 = Nothing

				Do While Value4 > 0
					await Task.Delay(delay)
					Value4 -= 1
				Loop
				Value4 = Nothing

			Finally
				IsBusy = False
			End Try
		End Sub

		Private ReadOnly m_clearValuesCommand As DelegateCommand
		Public ReadOnly Property ClearValuesCommand() As ICommand
			Get
				Return m_clearValuesCommand
			End Get
		End Property


		Public Event CloseRequested As EventHandler(Of EventArgs)
		''' <summary>
		''' Requests that the application be closed
		''' </summary>
		Public Sub RequestClose()
			OnCloseRequested(EventArgs.Empty)
		End Sub
		Protected Overridable Sub OnCloseRequested(ByVal e As EventArgs)
            RaiseEvent CloseRequested(Me, e)
        End Sub
		Private ReadOnly m_quitCommand As DelegateCommand

		Public ReadOnly Property QuitCommand() As ICommand
			Get
				Return m_quitCommand
			End Get
		End Property
	End Class
End Namespace