﻿Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks
Imports System.Windows.Threading

Namespace SolSoft.DataBinding.WpfDemo
	''' <summary>
	''' See MSDN Magazine November 2015 "Async From The Start"
	''' https://msdn.microsoft.com/en-us/magazine/mt620013.aspx
	''' </summary>
	Friend Class Program
		''' <summary>
		''' The main entry point for the application.
		''' </summary>
		<STAThread>
		Shared Sub Main()
			Dim app As App = New App With {.ShutdownMode = ShutdownMode.OnExplicitShutdown}
			'app.InitializeComponent(); //doesn't exist right now
			SynchronizationContext.SetSynchronizationContext(New DispatcherSynchronizationContext())

			Dim p As New Program()
			AddHandler p.ExitRequested, Sub(sender, e) app.Shutdown()

			Dim programStart As Task = p.StartAsync()
			HandleExceptions(programStart, app)

			app.Run()
		End Sub

        Private Shared Async Sub HandleExceptions(ByVal tsk As Task, ByVal wpfApplication As Application)
            Try
                Await Task.Yield() 'ensure this runs as a continuation
                Await tsk
            Catch ex As Exception
                'deal with exception, either with message box
                'or delegating to general exception handling logic you may have wired up 
                'e.g. to app.DispatcherUnhandledException and AppDomain.UnhandledException
                MessageBox.Show(ex.ToString())

                wpfApplication.Shutdown()
            End Try
        End Sub

        Private Sub New()
		End Sub

		Public Function StartAsync() As Task
			Dim viewModel As New MainViewModel()
			AddHandler viewModel.CloseRequested, AddressOf viewModel_CloseRequested

			Dim windowClosed As EventHandler = Sub(sender, e) viewModel.RequestClose()


			Dim mainForm As New MainWindow()
			AddHandler mainForm.Closed, windowClosed
			mainForm.ViewModel = viewModel
			mainForm.Show()

            'no async work to do in this case

            Return Task.FromResult(CType(Nothing, Object))
        End Function

		Public Event ExitRequested As EventHandler(Of EventArgs)
		Private Sub viewModel_CloseRequested(ByVal sender As Object, ByVal e As EventArgs)
			OnExitRequested(EventArgs.Empty)
		End Sub

		Protected Overridable Sub OnExitRequested(ByVal e As EventArgs)
            RaiseEvent ExitRequested(Me, e)
        End Sub
	End Class
End Namespace
