﻿Imports System.ComponentModel
Imports System.Diagnostics.CodeAnalysis
Imports System.Linq
Imports System.Text

Namespace SolSoft.DataBinding
	''' <summary>
	''' Helper base class for implementing <see cref="IRaisePropertyChanged"/>
	''' </summary>
	Public MustInherit Class BaseViewModel
		Implements IRaisePropertyChanged
		''' <summary>
		''' Constructs an instance of this class
		''' </summary>
		Protected Sub New()
		End Sub

		''' <summary>
		''' <see cref="INotifyPropertyChanged.PropertyChanged"/>
		''' </summary>
		Public Event PropertyChanged As PropertyChangedEventHandler Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged

		''' <summary>
		''' Raises the <see cref="PropertyChanged"/> event
		''' </summary>
		Protected Overridable Sub OnPropertyChanged(ByVal args As PropertyChangedEventArgs)
            RaiseEvent PropertyChanged(Me, args)

        End Sub

		''' <summary>
		''' Allows derived classes to use <see cref="IRaisePropertyChanged.RaisePropertyChanged"/>
		''' </summary>
		''' <param name="propertyName">Name of the property that has changed</param>
		<SuppressMessage("Microsoft.Design", "CA1030:UseEventsWhereAppropriate")>
		Protected Overridable Sub RaisePropertyChanged(ByVal propertyName As String)
			OnPropertyChanged(New PropertyChangedEventArgs(propertyName))
		End Sub
		''' <summary>
		''' <see cref="IRaisePropertyChanged.RaisePropertyChanged"/>
		''' </summary>
		Private Sub IRaisePropertyChanged_RaisePropertyChanged(ByVal propertyName As String) Implements IRaisePropertyChanged.RaisePropertyChanged
			Me.RaisePropertyChanged(propertyName)
		End Sub


	End Class
End Namespace
