﻿Imports System
Imports System.Diagnostics.CodeAnalysis
Imports System.Linq
Imports System.Linq.Expressions
Imports System.Text

Namespace SolSoft.DataBinding
    ''' <summary>
    ''' Contains additional methods for creating <see cref="DerivedNotifyProperty&lt;TValue&gt;"/>
    ''' </summary>
    Partial Public Module DerivedNotifyProperty
        ''' <summary>
        ''' Creates a derived notify property that changes when the depended-upon property changes
        ''' </summary>
        'Private Sub New()
        'End Sub
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        <System.Runtime.CompilerServices.Extension>
        Public Function CreateDerivedNotifyProperty(Of TDependedProperty1, TValue)(ByVal owner As IRaisePropertyChanged, ByVal derivedPropertyName As String, ByVal dependedProperty As IProperty(Of TDependedProperty1), ByVal getDerivedValueFunction As Func(Of TDependedProperty1, TValue)) As DerivedNotifyProperty(Of TValue)
            Return New DerivedNotifyProperty(Of TValue)(owner, derivedPropertyName, Function() getDerivedValueFunction(dependedProperty.Value), dependedProperty)
        End Function

        ''' <summary>
        ''' Creates a derived notify property that changes when any depended-upon property changes
        ''' </summary>
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        <System.Runtime.CompilerServices.Extension>
        Public Function CreateDerivedNotifyProperty(Of TDependedProperty1, TDependedProperty2, TValue)(ByVal owner As IRaisePropertyChanged, ByVal derivedPropertyName As String, ByVal dependedProperty1 As IProperty(Of TDependedProperty1), ByVal dependedProperty2 As IProperty(Of TDependedProperty2), ByVal getDerivedValueFunction As Func(Of TDependedProperty1, TDependedProperty2, TValue)) As DerivedNotifyProperty(Of TValue)
            Return New DerivedNotifyProperty(Of TValue)(owner, derivedPropertyName, Function() getDerivedValueFunction(dependedProperty1.Value, dependedProperty2.Value), dependedProperty1, dependedProperty2)
        End Function

        ''' <summary>
        ''' Creates a derived notify property that changes when any depended-upon property changes
        ''' </summary>
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        <System.Runtime.CompilerServices.Extension>
        Public Function CreateDerivedNotifyProperty(Of TDependedProperty1, TDependedProperty2, TDependedProperty3, TValue)(ByVal owner As IRaisePropertyChanged, ByVal derivedPropertyName As String, ByVal dependedProperty1 As IProperty(Of TDependedProperty1), ByVal dependedProperty2 As IProperty(Of TDependedProperty2), ByVal dependedProperty3 As IProperty(Of TDependedProperty3), ByVal getDerivedValueFunction As Func(Of TDependedProperty1, TDependedProperty2, TDependedProperty3, TValue)) As DerivedNotifyProperty(Of TValue)
            Return New DerivedNotifyProperty(Of TValue)(owner, derivedPropertyName, Function() getDerivedValueFunction(dependedProperty1.Value, dependedProperty2.Value, dependedProperty3.Value), dependedProperty1, dependedProperty2, dependedProperty3)
        End Function

        ''' <summary>
        ''' Creates a derived notify property that changes when any depended-upon property changes
        ''' </summary>
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        <System.Runtime.CompilerServices.Extension>
        Public Function CreateDerivedNotifyProperty(Of TDependedProperty1, TDependedProperty2, TDependedProperty3, TDependedProperty4, TValue)(ByVal owner As IRaisePropertyChanged, ByVal derivedPropertyName As String, ByVal dependedProperty1 As IProperty(Of TDependedProperty1), ByVal dependedProperty2 As IProperty(Of TDependedProperty2), ByVal dependedProperty3 As IProperty(Of TDependedProperty3), ByVal dependedProperty4 As IProperty(Of TDependedProperty4), ByVal getDerivedValueFunction As Func(Of TDependedProperty1, TDependedProperty2, TDependedProperty3, TDependedProperty4, TValue)) As DerivedNotifyProperty(Of TValue)
            Return New DerivedNotifyProperty(Of TValue)(owner, derivedPropertyName, Function() getDerivedValueFunction(dependedProperty1.Value, dependedProperty2.Value, dependedProperty3.Value, dependedProperty4.Value), dependedProperty1, dependedProperty2, dependedProperty3, dependedProperty4)
        End Function


    End Module

End Namespace
