﻿Imports System
Imports System.Diagnostics
Imports System.Diagnostics.CodeAnalysis
Imports System.Linq
Imports System.Linq.Expressions
Imports System.Text

Namespace SolSoft.DataBinding
    ''' <summary>
    ''' Property representing whether an object is currently busy.  Tracks the number of times it has been set to 'true' and 'false' and aggregates this into an effective 'true' or 'false' accordingly.
    ''' </summary>
    <DebuggerDisplay("{Value}")>
    Public Class IsBusyNotifyProperty
        Inherits NotifyProperty(Of Boolean)
        ''' <summary>
        ''' Creates a new <see cref="IsBusyNotifyProperty"/> with the given owner and name
        ''' </summary>
        ''' <param name="owner">Object that contains the property</param>
        ''' <param name="nameExpression">Expression used to determine the name of the property</param>
        ''' <returns>A completed <see cref="IsBusyNotifyProperty"/> object</returns>
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        Public Shared Function Create(ByVal owner As IRaisePropertyChanged, ByVal nameExpression As Expression(Of Func(Of Boolean))) As IsBusyNotifyProperty
            Return New IsBusyNotifyProperty(owner, nameExpression.GetName())
        End Function


        ''' <summary>
        ''' Creates a new <see cref="IsBusyNotifyProperty"/> with the given owner and name
        ''' </summary>
        ''' <param name="owner">Object that contains the property</param>
        ''' <param name="name">Name of this property within the parent object</param>
        Public Sub New(ByVal owner As IRaisePropertyChanged, ByVal name As String)
            MyBase.New(owner, name)

        End Sub


        Private m_count As Integer = 0 'number of times the property was set to true

        ''' <summary>
        ''' Sets a new value for this property; if the new value differs from the old value, a PropertyChanged event will be fired from its owner
        ''' </summary>
        ''' <param name="newValue">New value to be set</param>
        ''' <param name="overrideEqualityCheck">Whether or not to override the equality check.  If true, will raise the change events even if the value is unchanged</param>
        ''' <returns>True if the value was changed</returns>
        Public Overrides Function SetValue(ByVal newValue As Boolean, ByVal overrideEqualityCheck As Boolean) As Boolean
            If newValue Then
                m_count += 1
                If m_count = 1 OrElse overrideEqualityCheck Then 'changed from false to true (or equality check skipped)
                    MyBase.SetValue(True, overrideEqualityCheck)
                    Return True
                Else
                    Return False
                End If
            Else
                m_count -= 1
                If m_count = 0 OrElse overrideEqualityCheck Then 'changed from true to false (or equality check skipped)
                    MyBase.SetValue(False, overrideEqualityCheck)
                    Return True
                Else
                    Return False
                End If
            End If
        End Function
    End Class
End Namespace
