﻿Imports System
Imports System.Collections.Generic
Imports System.Diagnostics.CodeAnalysis
Imports System.Linq
Imports System.Linq.Expressions
Imports System.Text

Namespace SolSoft.DataBinding
    ''' <summary>
    ''' Contains convenience methods for creating <see cref="NotifyProperty"/> objects
    ''' </summary>
    Public Module NotifyProperty
        ''' <summary>
        ''' Creates a new notify property with the given owner and name
        ''' </summary>
        ''' <typeparam name="T">Type of the property's value</typeparam>
        ''' <param name="owner">Object that contains the property</param>
        ''' <param name="nameExpression">Expression used to determine the name of the property</param>
        ''' <returns>A <see cref="NotifyProperty"/> object encapsulating the owner's property</returns>
        'Private Sub New()
        'End Sub
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        <System.Runtime.CompilerServices.Extension>
        Public Function CreateNotifyProperty(Of T)(ByVal owner As IRaisePropertyChanged, ByVal nameExpression As Expression(Of Func(Of T))) As NotifyProperty(Of T)
            Return New NotifyProperty(Of T)(owner, nameExpression.GetName())
        End Function

        ''' <summary>
        ''' Creates a new notify property with the given owner, name, and equality comparer
        ''' </summary>
        ''' <typeparam name="T">Type of the property's value</typeparam>
        ''' <param name="owner">Object that contains the property</param>
        ''' <param name="nameExpression">Expression used to determine the name of the property</param>
        ''' <param name="itemChangedComparer">Comparer used to determine if the property value has changed</param>
        ''' <returns>A <see cref="NotifyProperty"/> object encapsulating the owner's property</returns>
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        <System.Runtime.CompilerServices.Extension>
        Public Function CreateNotifyProperty(Of T)(ByVal owner As IRaisePropertyChanged, ByVal nameExpression As Expression(Of Func(Of T)), ByVal itemChangedComparer As IEqualityComparer(Of T)) As NotifyProperty(Of T)
            Return New NotifyProperty(Of T)(owner, nameExpression.GetName(), itemChangedComparer)
        End Function

        ''' <summary>
        ''' Creates a new notify property with the given owner and name
        ''' </summary>
        ''' <typeparam name="T">Type of the property's value</typeparam>
        ''' <param name="owner">Object that contains the property</param>
        ''' <param name="nameExpression">Expression used to determine the name of the property</param>
        ''' <param name="initialValue">Initial value of the property</param>
        ''' <returns>A <see cref="NotifyProperty"/> object encapsulating the owner's property</returns>
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        <System.Runtime.CompilerServices.Extension>
        Public Function CreateNotifyProperty(Of T)(ByVal owner As IRaisePropertyChanged, ByVal nameExpression As Expression(Of Func(Of T)), ByVal initialValue As T) As NotifyProperty(Of T)
            Return New NotifyProperty(Of T)(owner, nameExpression.GetName(), initialValue)
        End Function

        ''' <summary>
        ''' Creates a new notify property with the given owner and name
        ''' </summary>
        ''' <typeparam name="T">Type of the property's value</typeparam>
        ''' <param name="owner">Object that contains the property</param>
        ''' <param name="nameExpression">Expression used to determine the name of the property</param>
        ''' <param name="initialValue">Initial value of the property</param>
        ''' <param name="itemChangedComparer">Comparer used to determine if the property value has changed</param>
        ''' <returns>A <see cref="NotifyProperty"/> object encapsulating the owner's property</returns>
        <SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")>
        <System.Runtime.CompilerServices.Extension>
        Public Function CreateNotifyProperty(Of T)(ByVal owner As IRaisePropertyChanged, ByVal nameExpression As Expression(Of Func(Of T)), ByVal initialValue As T, ByVal itemChangedComparer As IEqualityComparer(Of T)) As NotifyProperty(Of T)
            Return New NotifyProperty(Of T)(owner, nameExpression.GetName(), initialValue, itemChangedComparer)
        End Function
    End Module
End Namespace
