﻿Imports System
Imports System.Linq
Imports System.Linq.Expressions
Imports System.Text

Namespace SolSoft.DataBinding
    ''' <summary>
    ''' Extensions for getting names of code elements
    ''' </summary>
    Public Module ObjectNamingExtensions
        ''' <summary>
        ''' Gets the name of the member referenced by the given expression
        ''' </summary>
        ''' <typeparam name="T">Type of the return value of the expression</typeparam>
        ''' <param name="e">Expression, e.g. () => thisName will return thisName</param>
        <System.Runtime.CompilerServices.Extension>
        Public Function GetName(Of T)(ByVal e As Expression(Of Func(Of T))) As String
            If e Is Nothing Then
                Throw New ArgumentNullException(GetName(Function() e))
                'throw new ArgumentNullException(nameof(e))
            End If

            Dim member = TryCast(e.Body, MemberExpression)

            ' If the method gets a lambda expression 
            ' that is not a member access,
            ' for example, () => x + y, an exception is thrown.
            If member IsNot Nothing Then
                Return member.Member.Name
            Else
                Throw New ArgumentException("'" & e.ToString & "': is not a valid expression for this method")
            End If
        End Function

    End Module
End Namespace
